'''
-----------------------------------------------------------------------
File: auto-apply-pipeline.py
Creation Time: Nov 21st 2023 4:06 am
Author: Saurabh Zinjad
Developer Email: zinjadsaurabh1997@gmail.com
Copyright (c) 2023 Saurabh Zinjad. All rights reserved | GitHub: Ztrimus
-----------------------------------------------------------------------
'''
import os
import sys
import json

sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from utils.auto_apply_model import AutoApplyModel
from config import OPENAI_API_KEY
from utils.utils import read_json, read_file, write_file, write_json, job_doc_name, save_log, text_to_pdf
from utils.latex_ops import latex_to_pdf

output_dir = os.path.realpath("output")

def run_autoapply_pipeline(job_url: str, user_data_path: str = "master_data/user_profile.json"):
    try:
        if not len(job_url.strip()):
            print("Job URL is required.")
            return

        print("\n\nStarting Auto Apply Pipeline...")
        autoApply = AutoApplyModel(openai_key=OPENAI_API_KEY)

        print("\nFetching User data...")
        user_data = autoApply.user_data_extraction(user_data_path)

        print("\nExtracting Job Details...")
        job_details = autoApply.job_details_extraction(job_url)
        jd_path, resume_path, cv_path  = job_doc_name(job_details, output_dir)
        write_json(jd_path, job_details)

        print("\nGenerating Resume Details...")
        resume_details = autoApply.resume_builder(job_details, user_data)
        write_json(resume_path, resume_details)
        latex_to_pdf(resume_details, resume_path.replace(".json", ".pdf"))
        print("Resume PDF generated at: ", resume_path.replace(".json", ".pdf"))


        print("\nGenerating Cover Letter...")
        cv = autoApply.cover_letter_generator(job_details, user_data)
        write_file(cv_path, cv)
        print("Cover Letter generated at: ", cv_path)
        is_cv_to_pdf = input("Want cover letter as PDF? (y/n): ")
        if is_cv_to_pdf.strip().lower() == "y":
            text_to_pdf(cv, cv_path.replace(".txt", ".pdf"))
            print("Cover Letter PDF generated at: ", cv_path.replace(".txt", ".pdf"))
        
        print("Done!!!")
    except Exception as e:
        print(e)
        return None


'''
-----------------------------------------------------------------------
File: auto_apply_model.py
Creation Time: Nov 21st 2023 2:49 am
Author: Saurabh Zinjad
Developer Email: zinjadsaurabh1997@gmail.com
Copyright (c) 2023 Saurabh Zinjad. All rights reserved | GitHub: Ztrimus
-----------------------------------------------------------------------
'''
import os
import sys



sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))


import json
from utils.llm_models import ChatGPT
from utils.data_extraction import get_url_content, extract_text
from utils.utils import measure_execution_time, read_json
class AutoApplyModel:
    def __init__(self, openai_key: str):
        self.openai_key = openai_key
    
    def get_system_prompt(self, system_prompt_path: str):
        return open(system_prompt_path).read().strip()+"\n"

    def get_resume_to_json(self, pdf_path):
        system_prompt = self.get_system_prompt("../prompts/resume-extractor.txt")
        chat_gpt = ChatGPT(openai_api_key=self.openai_key, system_prompt=system_prompt)
        resume_text = extract_text(pdf_path)
        resume_text = chat_gpt.get_response(resume_text)
        resume_json = json.loads(resume_text)
        return resume_json
    
    @measure_execution_time
    def user_data_extraction(self, user_data_path: str):
        if os.path.splitext(user_data_path)[1] == '.pdf':
            return self.get_resume_to_json(user_data_path)
        else:
            return read_json(user_data_path)

    @measure_execution_time
    def job_details_extraction(self, url: str):
        """
        Extracts job details from the specified job URL.

        Args:
            url (str): The URL of the job posting.

        Returns:
            dict: A dictionary containing the extracted job details.
        """   

        system_prompt = self.get_system_prompt("../prompts/persona-job-llm.txt") + \
                        self.get_system_prompt("../prompts/extract-job-detail.txt")
        job_site_content = get_url_content(url)

        chat_gpt = ChatGPT(openai_api_key=self.openai_key, system_prompt=system_prompt)
        response = chat_gpt.get_response(job_site_content)
        job_details = json.loads(response)
        
        return job_details
    
    @measure_execution_time
    def resume_builder(self, job_details: dict, user_data: dict):
        system_prompt = self.get_system_prompt("../prompts/persona-job-llm.txt") + \
                        self.get_system_prompt("../prompts/generate-resume-details.txt")
        query = f"""Provided Job description delimited by triple backticks(```) and my resume or work information below delimited by triple dashes(---). ```{json.dumps(job_details)}``` ---{json.dumps(user_data)}---"""
        
        chat_gpt = ChatGPT(openai_api_key=self.openai_key, system_prompt=system_prompt)
        response = chat_gpt.get_response(query, expecting_longer_output=True)
        job_details = json.loads(response)
        return job_details
    
    @measure_execution_time
    def cover_letter_generator(self, job_details: dict, user_data: dict):
        system_prompt = self.get_system_prompt("../prompts/persona-job-llm.txt") + \
                        self.get_system_prompt("../prompts/generate-cover-letter.txt")
        query = f"""Provided Job description delimited by triple backticks(```) and \
                    my resume or work information below delimited by triple dashes(---).
                    ```
                    {json.dumps(job_details)}
                    ```

                    ---
                    {json.dumps(user_data)}
                    ---
                """
        
        chat_gpt = ChatGPT(openai_api_key=self.openai_key, system_prompt=system_prompt)
        cover_letter = chat_gpt.get_response(query, expecting_longer_output=True)
        return cover_letter


'''
-----------------------------------------------------------------------
File: LLM.py
Creation Time: Nov 1st 2023 1:40 am
Author: Amey Bhilegaonkar
Developer Email: abhilega@asu.edu
Copyright (c) 2023 Amey Bhilegaonkar. All rights reserved | GitHub: ameygoes
-----------------------------------------------------------------------
'''

import openai
import sys
import openai
sys.path.append('../')

from zlm.utils.data_extraction import get_url_content, extract_text
from zlm.utils.utils import read_file, write_json, ext

from zlm import config
from zlm.utils.llm_models import ChatGPT

resume_prompt = open("./prompts/resume-extractor.txt").read().strip()
# TODO: Consider 2 separate prompts for persona-job-llm and job-details-extractor
job_prompt = open("./prompts/job-detail-extractor.txt").read().strip()
compare_prompt = open("./prompts/compare-prompt.txt").read().strip()

def createGPTs():
    resumeGPT = ChatGPT(openai_api_key=config.OPENAI_API_KEY, system_prompt=resume_prompt)
    jobGPT = ChatGPT(openai_api_key=config.OPENAI_API_KEY, system_prompt=job_prompt)
    compareGPT = ChatGPT(openai_api_key=config.OPENAI_API_KEY, system_prompt=compare_prompt)
    return [resumeGPT, jobGPT, compareGPT]


resumeGPT, jobGPT, compareGPT = createGPTs()

# parse resume
pdf_path = '../input/user_resume.pdf'
resume_text = extract_text(pdf_path)
output_json_path = '../output/ameyBhilegaonkar.json'

# parse resume and save JSON file
pdf_parser.resume_text = resumeGPT.get_response(resume_text)
pdf_parser.save_to_json()

jobDetails = jobGPT.get_response(get_url_content("https://www.linkedin.com/jobs/view/3760365533"))
write_json("../output/job_details.json", jobDetails)

# compare resume and job details and craft resume
resumeContents = read_file("../output/ameyBhilegaonkar.json")
jobContents = read_file("../output/job_details.json")
compareContents = read_file("./prompts/compare-create-resume-from-jd.txt")
compareContents = compareContents.replace("<< RESUME >>", resumeContents).replace("<< DESCRIPTION >>", jobContents)

print(compareContents)
finalResumePrompt = compareGPT.get_response(compareContents)
write_json("../output/final-resume.json", finalResumePrompt)